#include <EEPROM.h>
#include <LiquidCrystal.h>
LiquidCrystal lcd(2, 3, 4, 5, 6, 7);

#define SW1 A2  // Press time
#define SW2 A3  // Vac time
#define SW3 A0  // Mode
#define SW4 A1  // Delay time
#define Trig 11  // Trigger switch
#define VOut 12  // Vaccum Valve out
#define POut 13  // Pressure valve out 13

int p=0,ptime,v=0,vtime,m=0,mode,d=5,dtime,trg=0;

void setup() {      
  //  Serial.begin(9600);
  pinMode(SW1, INPUT_PULLUP);
  digitalWrite(SW1, HIGH);  
  pinMode(SW2, INPUT_PULLUP);
  digitalWrite(SW2, HIGH);  
  pinMode(SW3, INPUT_PULLUP);
  digitalWrite(SW4, HIGH);  
  pinMode(SW4, INPUT_PULLUP);
  digitalWrite(SW4, HIGH);  
  pinMode(Trig, INPUT_PULLUP);  
  digitalWrite(Trig, HIGH);  
  pinMode(POut, OUTPUT);  
  digitalWrite(POut, LOW); 
  pinMode(VOut, OUTPUT);  
  digitalWrite(VOut, LOW); 

  ptime=EEPROM.read(10);
  vtime=EEPROM.read(11);
  mode=EEPROM.read(12);
  dtime=EEPROM.read(13);

  if (ptime > 99) ptime=10; // Limit 
  if (vtime > 99) vtime=10;
  if (mode > 16) mode=1;
  if (dtime > 39) dtime=10;
  if (dtime < 5) dtime=10;

  p=ptime;
  v=vtime;  
  m=mode;
  d=dtime;

  lcd.begin(8, 2);
  lcd.setCursor(0, 0);
  lcd.print("Dysp_V11");
  lcd.setCursor(0, 1);
  lcd.print("traxman ");
  delay(1000);
  lcd.clear();

  if (p < 10){
    lcd.setCursor(0, 0);
    lcd.print("0.");        
    lcd.setCursor(2, 0);   
    lcd.print(round(p));       
  }
  else{
    lcd.setCursor(0, 0);
    lcd.print(p/10);
    lcd.setCursor(1, 0);
    lcd.print(".");
    lcd.setCursor(2, 0);
    lcd.print(round(p%10));
  } 
  if (v < 10){
    lcd.setCursor(0, 1);
    lcd.print("0.");        
    lcd.setCursor(2, 1);   
    lcd.print(round(v));       
  }
  else{
    lcd.setCursor(0, 1);
    lcd.print(v/10);
    lcd.setCursor(1, 1);
    lcd.print(".");
    lcd.setCursor(2, 1);
    lcd.print(round(v%10));
  }

  lcd.setCursor(5, 0);
  if (m == 0)
  {
    lcd.print("MAN");
  }
  if (m == 1)
  {
    lcd.print("IMP");
  }
  if (m > 1)
  {
    lcd.print("S");
    if (m < 10)
    {
      lcd.setCursor(6, 0);
      lcd.print(" ");
      lcd.setCursor(7, 0);
      lcd.print(m);
    }
    else
    {
      lcd.setCursor(6, 0);
      lcd.print(m);
    }
  } 

  if (d < 10){
    lcd.setCursor(5, 1);
    lcd.print("0.");        
    lcd.setCursor(7, 1);   
    lcd.print(round(d));       
  }
  else{
    lcd.setCursor(5, 1);
    lcd.print(d/10);
    lcd.setCursor(6, 1);
    lcd.print(".");
    lcd.setCursor(7, 1);
    lcd.print(round(d%10));
  }
}

void set_mode()
{
  if (digitalRead(SW1) == 0)  // Press time value 0-99 (0.0-9.9s)
  { 
    if (p < 10){
      lcd.setCursor(0, 0);
      lcd.print("0.");        
      lcd.setCursor(2, 0);   
      lcd.print(round(p));       
    }
    else{
      lcd.setCursor(0, 0);
      lcd.print(p/10);
      lcd.setCursor(1, 0);
      lcd.print(".");
      lcd.setCursor(2, 0);
      lcd.print(round(p%10));
    }
    ptime=p;
    p++; 
    if (p > 99) p=0;  // p =99 max Press time (0.0-9.9s)
  }

  if (digitalRead(SW2) == 0)  // Vac time value 0-99 (0.0-9.9s)
  { 
    if (v < 10){
      lcd.setCursor(0, 1);
      lcd.print("0.");        
      lcd.setCursor(2, 1);   
      lcd.print(round(v));       
    }
    else{
      lcd.setCursor(0, 1);
      lcd.print(v/10);
      lcd.setCursor(1, 1);
      lcd.print(".");
      lcd.setCursor(2, 1);
      lcd.print(round(v%10));
    }
    vtime=v;
    v++; 
    if (v > 99) v=0;  // p =99 max Vac time 0.0-9.9s
  }

  if (digitalRead(SW3) == 0)  // Mode 0-MAN, 1-IMP, 2-SEQ 2..16
  { 
    lcd.setCursor(5, 0);
    if (m == 0)
    {
      lcd.print("MAN");
    }
    if (m == 1)
    {
      lcd.print("IMP");
    }
    if (m > 1)
    {
      lcd.print("S");
      if (m < 10)
      {
        lcd.setCursor(6, 0);
        lcd.print(" ");
        lcd.setCursor(7, 0);
        lcd.print(m);
      }
      else
      {
        lcd.setCursor(6, 0);
        lcd.print(m);
      }
    } 
    mode=m;
    m++;  
    if (m > 16) m=0;  
  }

  if (digitalRead(SW4) == 0)  // Delay time value 0-30 (0.0-3.9s)
  {                           
    if (d < 10){
      lcd.setCursor(5, 1);
      lcd.print("0.");        
      lcd.setCursor(7, 1);   
      lcd.print(round(d));       
    }
    else{
      lcd.setCursor(5, 1);
      lcd.print(d/10);
      lcd.setCursor(6, 1);
      lcd.print(".");
      lcd.setCursor(7, 1);
      lcd.print(round(d%10));
    }
    dtime=d;
    d++; 
    if (d > 39)  // d =39 max Delay time 0.5-3.9s
    {
      d=5;
    }
  }
}

void store_cfg()
{
  if (ptime != EEPROM.read(10)) 
  {
    EEPROM.write(10, ptime) ;
  }
  if (vtime != EEPROM.read(11)) 
  {
    EEPROM.write(11, vtime) ;
  }
  if (mode != EEPROM.read(12)) 
  {
    EEPROM.write(12, mode) ;
  }
  if (dtime != EEPROM.read(13)) 
  {
    EEPROM.write(13, dtime) ;
  }
}

void loop() {

  set_mode();
  store_cfg();
  delay(200); 

  /*  Serial.print(ptime);  
   Serial.print(" ");
   Serial.print(vtime);  
   Serial.print(" ");
   Serial.print(mode);  
   Serial.print(" ");
   Serial.println(dtime);
   */

  if (mode == 0)
  {
    // MAN + VAC (vtime)
    if (digitalRead(Trig) == 0)
    {
      digitalWrite(POut, HIGH); //press valve on   
      trg=1;   
    }
    else
    {
      digitalWrite(POut, LOW);  //press valve off       
      if (trg > 0)
      {
        if (vtime > 0)
        {
          digitalWrite(VOut, HIGH); //vac valve on       
          for (int i=0; i< vtime; i++)
          {
            delay(100);
          }
          digitalWrite(VOut, LOW); //vac valve on after dtime
        }
        trg=0;
      }
    }
  }
  else if (mode == 1)
  {
    // ptime + vtime
    if (digitalRead(Trig) == 0)
    {
      digitalWrite(POut, HIGH); //press valve on   
      for (int i=0; i< ptime; i++)
      {
        delay(100);
      }
      digitalWrite(POut, LOW); //press valve on after ptime    
      digitalWrite(VOut, HIGH); //vac valve on   
      for (int i=0; i< vtime; i++)
      {
        delay(100);
      }
      digitalWrite(VOut, LOW); //vac valve on after vtime  
    }
  }
  else
  {
    // sequence  + m x dtime
    if (digitalRead(Trig) == 0)
    {
      for (int j =0; j< mode; j++)
      {
        digitalWrite(POut, HIGH); //press valve on   
        for (int i=0; i< ptime; i++)
        {
          delay(100);
        }
        digitalWrite(POut, LOW); //press valve on after ptime    
        digitalWrite(VOut, HIGH); //vac valve on   
        for (int i=0; i< vtime; i++)
        {
          delay(100);
        }
        digitalWrite(VOut, LOW); //vac valve on after vtime  

        for (int k=0; k< dtime; k++) // delay dtime
        {
          delay(100);
        }      
      }
    }
  }
}






